/* file to #include to validate a WIND ticket */

#ifndef _SSL_CLIENT_H
#define _SSL_CLIENT_H


#ifdef __cplusplus
extern "C" {
#endif 


/* Function to perform WIND validation; return 1 on good ticket, 0 on
   failure; UNI/username will be put into 'outbuf'.  Error message
   returned in 'error' if an error occurs
   
   See README for more.
 */
extern int WIND_validate(const char *ticket, char *service,
			 char *outbuf, int outbuflen, char * error);

/* Function to add a trusted CA certificate to the list of trusted
   certs.  Without a trusted cert, we don't trust anyone so we can't
   use WIND.  'path' should be the full pathname of the trusted
   cert. */
extern void WIND_addTrustedCA(const char * path);

#ifdef __cplusplus
}
#endif 

#endif /* _SSL_CLIENT_H */
