/* Configurable WIND information */


/* ------------------------------------------------------------------------ */
/* first, stuff that's static regardless of server */

/* the list of ca's to trust
   THIS MECHANISM IS NO LONGER USED.
   the list is maintained via the WIND_addTrustedCA() function, and is
   initially empty */
/*
static char *trusted_ca[] = {
    "/src/web/wind/aux/sslvalidator/CrenCA.pem",
    "/src/web/wind/aux/sslvalidator/CrenSignedColumbiaCA.pem",
    NULL
};
*/

#define CAS_METHOD "GET"
#define CAS_VALIDATE "/sec-cgi-bin/wind/validate"
#define CAS_PROT "HTTP/1.0"


/* ------------------------------------------------------------------------ */
/* next, server-dependent stuff */

#ifdef TEST_SERVER

/* This is the test server */
#define CAS_LOGIN_URL "https://doorstop.cc.columbia.edu:8443/sec-cgi-bin/wind/login"
#define CAS_HOST "doorstop.cc.columbia.edu"
#define CAS_PORT 8443 

#else

/* This is the production server */
#define CAS_LOGIN_URL "https://www1.columbia.edu/sec-cgi-bin/wind/login"
#define CAS_HOST "www1.columbia.edu"
#define CAS_PORT 443

#endif

